package services.ejb;

import java.math.BigDecimal;
import java.util.List;
import java.util.Random;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.ejb3.annotation.TransactionTimeout;
import org.jboss.logmanager.Level;
import org.jboss.logmanager.Logger;

import services.ejb.CustomerManager;
import services.entities.Customer;
import services.entities.PricingMethod;
import services.entities.Region;
import services.exceptions.CreateDataException;

@Stateless
public class CustomerManagerBean implements CustomerManager {

	private static Logger logger = Logger.getLogger(CustomerManagerBean.class.toString());
	
	@EJB
	DistributionCenterManager distributionCenterManager;
	
	@PersistenceContext (unitName="services")
	protected EntityManager entityManager;
	
	@PersistenceContext(unitName="batch-services")
	protected EntityManager batchEntityManager;

	public Customer findByPrimaryKey(long customerId) {

		Query query = entityManager.createNamedQuery("selectCustomerByPrimaryKey");

		query.setParameter("customerId", customerId);

		Customer customer = (Customer) query.getSingleResult();

		return customer;

	}

	public int numberOfCustomers() {

		Long numberOfCustomers = (Long) entityManager.createQuery("select count(*) from Customer").getSingleResult();
		
		return numberOfCustomers.intValue(); 

	}

	@SuppressWarnings("unchecked")
	public List<Customer> findAllCustomers() {

		Query query = entityManager.createNamedQuery("selectAllCustomers");
		
		return query.getResultList();

	}

	public void createCustomer(Customer customer) {

		entityManager.persist(customer);

	}

	public void createPricingMethod(PricingMethod pricingMethod) {

		entityManager.persist(pricingMethod);

	}

	@TransactionTimeout(600)
	public void createCustomers(int numberOfCustomers, int batchSize)
		throws CreateDataException {
		
		String[] streets = {" Soak Testing", " Scalability Testing", " Availability Testing"
				, " Performance Testing"
		};
		
		String[] streetSuffixes = {" Dr.", " St.", " Road", " Way", " Court", " Circle"
				, " Blvd."
		};
		
		boolean priceMethod = true;
		
		int rowsToFlush = 0, x = 0, numberOfRegions = 0;
		
		BigDecimal percent = new BigDecimal("0.2250");
		
		Random streetNumber = new Random(System.currentTimeMillis());
		Random streetName   = new Random(System.currentTimeMillis());
		Random streetSuffix = new Random(System.currentTimeMillis());
		
		// Find all the Regions for use in creating the customers
		
		List<Region> regions = distributionCenterManager.findAllRegions();
		
		if(regions == null || regions.size() == 0) {
			throw new CreateDataException("There were no Regions in the Regions table to build customers from!");
		}
		
		numberOfRegions = regions.size();
		
		Customer customer = null;
		PricingMethod pricingMethod = null;
		
		for(int i = 0; i < numberOfCustomers; i++) {
			customer = new Customer();
			
			customer.setCustomerId((i+1));
			customer.setCustomerName("Test " + (i+1));
			customer.setAddressLine1(streetNumber.nextInt(9999) + streets[streetName.nextInt(4)] + streetSuffixes[streetSuffix.nextInt(7)]);
			customer.setCity(regions.get(x).getCity());
			customer.setState(regions.get(x).getState());
			customer.setZipCode(regions.get(x).getPrimaryKey().getZipCode());
			customer.setZipCodePlusFour(regions.get(x).getZipCodePlusFour());
			customer.setDistributionCenterId(regions.get(x).getDistributionCenter().getDistributionCenterId());
			
			pricingMethod = new PricingMethod();
			
			pricingMethod.setCustomerId(customer.getCustomerId());
			pricingMethod.setPercent(percent);
			
			if(priceMethod) {
				pricingMethod.setCostPlus(priceMethod);
				pricingMethod.setListLess(false);
				priceMethod = false;
			} else {
				pricingMethod.setCostPlus(priceMethod);
				pricingMethod.setListLess(true);
				priceMethod = true;
			}
			
			customer.setPricingMethod(pricingMethod);
			
			batchEntityManager.persist(customer);
			
			rowsToFlush++;
			
			if(rowsToFlush == batchSize) {
				batchEntityManager.flush();
				rowsToFlush = 0;
				batchEntityManager.clear();
				if(logger.isLoggable(Level.TRACE)) {
					logger.log(Level.TRACE, "Just flushed " + batchSize + " rows to the database.");
					logger.log(Level.TRACE, "Total rows flushed is " + (i+1));
				}
			}
			
			x++;
			
			if(x == numberOfRegions) {
				x = 0;
			}
		}
		
		return;

	}

}
