/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package services.ejb;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

//import org.jboss.ejb3.annotation.Clustered;

import services.entities.Order;

//@Clustered
@Stateful
public class OrderInquiryBean implements OrderInquiry, Serializable {
   
   
   /** The serialVersionUID */
   private static final long serialVersionUID = -9113338082170911949L;
   
   private static final int pagination = 20;
   private static final int maxResults = 500;
   private List<Order> orders;
   private int pageNumber = 1;
   
   @PersistenceContext(unitName="services")
   protected EntityManager entityManager;
   
   public int getPaginationForInquiries() {
   
      return pagination;
      
   }
   
   @SuppressWarnings("unchecked")
   //@TransactionAttribute(TransactionAttributeType.NEVER)
   public List<Order> findOrdersByCustomer(long customerId) {
      
      List<Order> pageOfOrders = new ArrayList<Order>(pagination);
      
      if (orders == null) {
    	  orders = entityManager
    	  	.createNamedQuery("selectOrdersByCustomerDesc")
    	  	.setParameter("customerId", customerId)
    	  	.setMaxResults(maxResults)
    	  	.getResultList();
      }
      
      int startOrder = 0;
      int endOrder = pagination - 1;
      
      if (pageNumber > 1) {
         endOrder = pagination * pageNumber - 1;
         startOrder = endOrder - pagination + 1;
      }
      
      int i = 0;
      
      for (Order order: orders) {
         if (i >= startOrder && i <= endOrder) {
            pageOfOrders.add(order);
         }
         i++;
      }
      
      pageNumber++;
      
      return pageOfOrders;
      
   }
   
   public void reset() {
	   
	   orders     = null;
	   pageNumber = 1;
	   
	   return;
	   
   }
   
   @Remove
   public void cancelSession() {
      
   }

}