package services.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.QueryHint;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@Entity
@NamedQueries(value={@NamedQuery(name="selectCustomerByPrimaryKey", query="from Customer where customerId = :customerId", hints={@QueryHint(name = "org.hibernate.cacheable", value = "true")})
		, @NamedQuery(name="selectAllCustomers", query="from Customer order by customerId")
		})
public class Customer implements Serializable {

	private static final long serialVersionUID = -7155977318939198606L;

	private long customerId;
	private String customerName;
	private Address customerAddress;
	private List<Order> orders;
	private PricingMethod pricingMethod;
	private long distributionCenterId;
	private DistributionCenter servedByDc;

	public Customer() {

		customerAddress = new Address();
		orders = new ArrayList<Order>();

	}

	@Id()
	public long getCustomerId() {

		return customerId;

	}

	public void setCustomerId(long customerId) {

		this.customerId = customerId;

	}

	public String getCustomerName() {

		return customerName;

	}

	public void setCustomerName(String customerName) {

		this.customerName = customerName;

	}

	public String getAddressLine1() {

		return customerAddress.getAddressLine1();

	}

	public void setAddressLine1(String addressLine1) {

		customerAddress.setAddressLine1(addressLine1);

	}

	@Column(nullable = true)
	public String getAddressLine2() {

		return customerAddress.getAddressLine2();

	}

	public void setAddressLine2(String addressLine2) {

		customerAddress.setAddressLine2(addressLine2);

	}

	public String getCity() {

		return customerAddress.getCity();

	}

	public void setCity(String city) {

		customerAddress.setCity(city);

	}

	public String getState() {

		return customerAddress.getState();

	}

	public void setState(String state) {

		customerAddress.setState(state);

	}

	public int getZipCode() {

		return customerAddress.getZipCode();

	}

	public void setZipCode(int zipCode) {

		customerAddress.setZipCode(zipCode);

	}

	public int getZipCodePlusFour() {

		return customerAddress.getZipCodePlusFour();

	}

	public void setZipCodePlusFour(int zipCodePlusFour) {

		customerAddress.setZipCodePlusFour(zipCodePlusFour);

	}

	@OneToMany(mappedBy = "customer", cascade = { CascadeType.ALL }, fetch = FetchType.LAZY)
	public List<Order> getOrders() {

		return orders;

	}

	public void setOrders(List<Order> orders) {

		this.orders = orders;

	}

	@OneToOne(mappedBy = "customer", cascade = { CascadeType.ALL }, fetch = FetchType.LAZY)
	@JoinColumn(name = "CustomerId", insertable = false, updatable = false)
	//@Cache(usage=CacheConcurrencyStrategy.READ_ONLY)
	public PricingMethod getPricingMethod() {

		return pricingMethod;

	}

	public void setPricingMethod(PricingMethod pricingMethod) {

		this.pricingMethod = pricingMethod;

	}

	public long getDistributionCenterId() {

		return distributionCenterId;

	}

	public void setDistributionCenterId(long distributionCenterId) {

		this.distributionCenterId = distributionCenterId;

	}

	@ManyToOne()
	@JoinColumn(name = "DistributionCenterId", insertable = false, updatable = false)
	public DistributionCenter getServedByDc() {

		return servedByDc;

	}

	public void setServedByDc(DistributionCenter servedByDc) {

		this.servedByDc = servedByDc;

	}

	public boolean equals(Object o) {

		if (this == o) {
			return true;
		}

		if (o == null || getClass() != o.getClass()) {
			return false;
		}

		final Customer customer = (Customer) o;

		if (!(customerId == customer.customerId)) {
			return false;
		}

		if (!(customerName.equals(customer.customerName))) {
			return false;
		}

		if (!(customerAddress.equals(customer.customerAddress))) {
			return false;
		}

		return true;

	}

	public int hashCode() {

		int hashValue;

		hashValue = Long.toString(customerId).hashCode()
				+ customerName.hashCode() + customerAddress.hashCode();

		return hashValue;

	}

}
