/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package services.entities;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@Entity
@NamedQueries(value = {@NamedQuery(name="selectAllTheRegions", query="from Region")
		, @NamedQuery(name="selectAllTheDistributionCenters", query="from DistributionCenter")
		})
public class DistributionCenter implements Serializable {

   /** The serialVersionUID */
   private static final long serialVersionUID = -7147495259971700642L;
   private long distributionCenterId;
   private String description;
   private Address location;
   private Set<Region> regionsServed;
   private Set<Customer> customersServed;
   
   public DistributionCenter() {
      
      location = new Address();

   }
   
   @Id
   public long getDistributionCenterId() {
      
      return distributionCenterId;
      
   }
   
   public void setDistributionCenterId(long distributionCenterId) {
      
      this.distributionCenterId = distributionCenterId;
      
   }
   
   public String getDescription() {
      
      return description;
      
   }
   
   public void setDescription(String description) {
      
      this.description = description;
      
   }
   
   public String getAddressLine1() {
      
      return location.getAddressLine1();
      
   }
   
   public void setAddressLine1(String addressLine1) {
      
      location.setAddressLine1(addressLine1);
      
   }
   
   @Column(nullable=true)
   public String getAddressLine2() {
      
      return location.getAddressLine2();
      
   }
   
   public void setAddressLine2(String addressLine2) {
      
      location.setAddressLine2(addressLine2);
      
   }
   
   public String getCity() {
      
      return location.getCity();
      
   }
   
   public void setCity(String city) {
      
      location.setCity(city);
      
   }
   
   public String getState() {
      
      return location.getState();
      
   }
   
   public void setState(String state) {
      
      location.setState(state);
      
   }
   
   public int getZipCode() {
      
      return location.getZipCode();
      
   }
   
   public void setZipCode(int zipCode) {
      
      location.setZipCode(zipCode);
      
   }
   
   public int getZipCodePlusFour() {
      
      return location.getZipCodePlusFour();
      
   }
   
   public void setZipCodePlusFour(int zipCodePlusFour) {
      
      location.setZipCodePlusFour(zipCodePlusFour);
      
   }
   
   @OneToMany(mappedBy="distributionCenter", cascade={CascadeType.ALL}, fetch=FetchType.LAZY)
   public Set<Region> getRegionsServed() {
      
      return regionsServed;
       
   }
   
   public void setRegionsServed(Set<Region> regionsServed) {
      
       this.regionsServed = regionsServed;
       
   }
   
   @OneToMany(mappedBy="servedByDc", cascade={CascadeType.ALL}, fetch=FetchType.LAZY)
   public Set<Customer> getCustomersServed() {
      
      return customersServed;
      
   }
   
   public void setCustomersServed(Set<Customer> customersServed) {
      
      this.customersServed = customersServed;
      
   }
   
   public boolean equals(Object o) {
      
      if (this == o) {
         return true;
      }
      
      if (o == null || getClass() != o.getClass()) {
         return false;
      }
      
      final DistributionCenter distributionCenter = (DistributionCenter) o;
      
      if (!(distributionCenterId == distributionCenter.distributionCenterId)) {
         return false;
      }
      
      if (!(location.equals(distributionCenter.location))) {
         return false;
      }
      
      return true;
      
   }
   
   public int hashCode() {
      
      int hashValue;
      
      hashValue = Long.toString(distributionCenterId).hashCode()
         + location.hashCode();
      
      return hashValue;
      
   }

}
