package services.entities;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.GenericGenerator;

@Cache(usage = CacheConcurrencyStrategy.NONE)
@Entity
@NamedQueries(value={ @NamedQuery(name="selectOrdersByCustomerDesc", query="from Order where CustomerId = :customerId order by OrderDate desc")
		, @NamedQuery(name="oldestOrderDateByProductAndDistributionCenter"
			, query="select MIN(o.orderDate) from Order as o JOIN o.orderLines as ol where o.orderId = ol.orderLinePK.orderId and ol.productId = :productId and o.distributionCenterId = :distributionCenterId")
		, @NamedQuery(name="newestOrderDateByProductAndDistributionCenter"
			, query="select MAX(o.orderDate) from Order as o JOIN o.orderLines as ol where o.orderId = ol.orderLinePK.orderId and ol.productId = :productId and o.distributionCenterId = :distributionCenterId")
		, @NamedQuery(name="demandByProductAndDistributionCenter"
			, query="select SUM(ol.quantity) from Order as o JOIN o.orderLines as ol where o.orderId = ol.orderLinePK.orderId and ol.productId = :productId and o.distributionCenterId = :distributionCenterId")
		})
public class Order implements Serializable {
	
	private static final long serialVersionUID = -5721274468042182248L;
	
	private String orderId;
	private long customerId;
	private long distributionCenterId;
	private Address shippingAddress;
	private BigDecimal totalOrderAmount;
	private Date orderDate;
	private List<OrderLine> orderLines;
	private Customer customer;
	
	public Order() {

		shippingAddress  = new Address();
		totalOrderAmount = new BigDecimal(0);
		
	}
	
	@Id
	@GeneratedValue(generator="system-uuid")
	@GenericGenerator(name = "system-uuid", strategy = "uuid2")
	public String getOrderId () {
		
		return orderId;
		
	}
	
	public void setOrderId(String orderId) {
		
		this.orderId = orderId;
		
	}
	
	public long getCustomerId() {
		
		return customerId;
		
	}
	
	public void setCustomerId(long customerId) {
		
		this.customerId = customerId;
		
	}
	
	public long getDistributionCenterId() {
		
		return distributionCenterId;
		
	}
	
	public void setDistributionCenterId(long distributionCenterId) {
		
		this.distributionCenterId = distributionCenterId;
		
	}
	
	public String getShippingAddressLine1() {
		
		return shippingAddress.getAddressLine1();
		
	}
	
	public void setShippingAddressLine1(String shippingAddressLine1) {
		
		shippingAddress.setAddressLine1(shippingAddressLine1);
		
	}
	
	public String getShippingAddressLine2() {
		
		return shippingAddress.getAddressLine2();
		
	}
	
	public void setShippingAddressLine2(String shippingAddressLine2) {
		
		shippingAddress.setAddressLine2(shippingAddressLine2);
		
	}
	
	public String getShippingCity() {
		
		return shippingAddress.getCity();
		
	}
	
	public void setShippingCity(String shippingCity) {
		
		shippingAddress.setCity(shippingCity);
		
	}
	
	public String getShippingState() {
		
		return shippingAddress.getState();
		
	}
	
	public void setShippingState(String shippingState) {
		
		shippingAddress.setState(shippingState);
		
	}
	
	public int getShippingZipCode() {
		
		return shippingAddress.getZipCode();
		
	}
	
	public void setShippingZipCode(int shippingZipCode) {
		
		shippingAddress.setZipCode(shippingZipCode);
		
	}
	
	public int getShippingZipCodePlusFour() {
		
		return shippingAddress.getZipCodePlusFour();
		
	}
	
	public void setShippingZipCodePlusFour(int shippingZipCodePlusFour) {
		
		shippingAddress.setZipCodePlusFour(shippingZipCodePlusFour);
		
	}
	
	public BigDecimal getTotalOrderAmount() {
		
		return totalOrderAmount;
		
	}
	
	public void setTotalOrderAmount(BigDecimal totalOrderAmount) {
		
		this.totalOrderAmount = totalOrderAmount;
		
	}
	
	public Date getOrderDate() {
		
		return orderDate;
		
	}
	
	public void setOrderDate(Date orderDate) {
		
		this.orderDate = orderDate;
		
	}
	
	@OneToMany(mappedBy="order", cascade={CascadeType.ALL}, fetch=FetchType.EAGER)
	public List<OrderLine> getOrderLines() {
		
		return orderLines;
		
	}
	
	public void setOrderLines(List<OrderLine> orderLines) {
		
		this.orderLines = orderLines;
		
	}
   
   @ManyToOne()
   @JoinColumn(name="CustomerId", insertable=false, updatable=false)
   public Customer getCustomer() {
      
      return customer;
      
   }
   
   public void setCustomer(Customer customer) {
      
      this.customer = customer;
      
   }
	
	public boolean equals(Object o) {
		
		if (this == o) {
			return true;
		}
		
		if (o == null || getClass() != o.getClass()) {
			return false;
		}
		
		final Order order = (Order) o;
		
		if (!(orderId == order.orderId)) {
			return false;
		}
		
		if (!(customerId == order.customerId)) {
			return false;
		}
		
		if (!shippingAddress.equals(order.shippingAddress)) {
			return false;
		}
		
		if (!totalOrderAmount.equals(order.totalOrderAmount)) {
			return false;
		}
		
		if (!orderLines.equals(order.orderLines)) {
			return false;
		}
		
		return true;
		
	}
	
	public int hashCode() {
		
		int hashValue;
		
		hashValue = orderId.hashCode()
			+ Long.toString(customerId).hashCode()
			+ shippingAddress.hashCode()
			+ totalOrderAmount.hashCode()
			+ orderLines.hashCode();
		
		return hashValue;
		
	}

}
