package services.web;

import java.io.IOException;
import java.io.PrintWriter;

import javax.ejb.EJB;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.logmanager.Level;
import org.jboss.logmanager.Logger;

import com.oreilly.servlet.MultipartResponse;

import services.ejb.CustomerManager;
import services.ejb.DistributionCenterManager;
import services.ejb.OrderManager;
import services.ejb.ProductManager;
import services.exceptions.CreateDataException;

public class CreateDataServlet extends HttpServlet {

	private static final long serialVersionUID = 8476304108777385279L;
	private static Logger logger = Logger.getLogger(CreateDataServlet.class.toString());
	
	@EJB(lookup="java:app/OrderManagerEJB/DistributionCenterManagerBean!services.ejb.DistributionCenterManager")
	private DistributionCenterManager distributionCenterManager;
	
	@EJB(lookup="java:app/OrderManagerEJB/CustomerManagerBean!services.ejb.CustomerManager")
	private CustomerManager customerManager;
	
	@EJB(lookup="java:app/OrderManagerEJB/ProductManagerBean!services.ejb.ProductManager")
	private ProductManager productManager;
	
	@EJB(lookup="java:app/OrderManagerEJB/OrderManagerBean!services.ejb.OrderManager")
	private OrderManager orderManager = null;
	
	public CreateDataServlet() {
		
		super();
		
	}
	
	public void init(ServletConfig config) throws ServletException {
	
		super.init(config);
		
		return;
		
	}
	
	public void doGet(HttpServletRequest request
			, HttpServletResponse response) throws ServletException, IOException {
		
		//  Send the client the form to fill in for the data parameters
		
		PrintWriter out = response.getWriter();
		
		response.setContentType("text/html");
		
		out.println("<html><head><title>");
		out.println("Set the Date Parameters!</title></head>");
		out.println("<P STYLE=\"margin-top: 0.17in; page-break-after: avoid\"><FONT FACE=\"Liberation Sans, sans-serif\"><FONT SIZE=4>Data Parameters</FONT></FONT></P>");
		out.println("<FORM NAME=\"DataParameters\" ACTION=\"CreateData\" METHOD=\"POST\">");
		out.println("<TABLE WIDTH=100% BORDER=0 CELLPADDING=0 CELLSPACING=0>");
		out.println("<COL WIDTH=89*>");
		out.println("<COL WIDTH=167*>");
		out.println("<TR VALIGN=TOP>");
		out.println("<FORM NAME=\"Standard\">");
		out.println("<TD WIDTH=35%>");
		out.println("<P><FONT FACE=\"Liberation Sans, sans-serif\">Number of Customers:</FONT></P>");
		out.println("</TD>");
		out.println("<TD WIDTH=65%>");
		out.println("<P><INPUT TYPE=TEXT NAME=\"numberOfCustomers\" SIZE=33 STYLE=\"width: 2.87in; height: 0.3in\"></P>");
		out.println("</TD>");
		out.println("</TR>");
		out.println("<TR VALIGN=TOP>");
		out.println("<TD WIDTH=35%>");
		out.println("<P><FONT FACE=\"Liberation Sans, sans-serif\">Number of Products:</FONT></P>");
		out.println("</TD>");
		out.println("<TD WIDTH=65%>");
		out.println("<P><INPUT TYPE=TEXT NAME=\"numberOfProducts\" SIZE=33 STYLE=\"width: 2.87in; height: 0.3in\"></P>");
		out.println("</TD>");
		out.println("</TR>");
		out.println("<TR VALIGN=TOP>");
		out.println("<TD WIDTH=35%>");
		out.println("<P><FONT FACE=\"Liberation Sans, sans-serif\">Batch Insert Size:</FONT></P>");
		out.println("</TD>");
		out.println("<TD WIDTH=65%>");
		out.println("<P><INPUT TYPE=TEXT NAME=\"batchSize\" SIZE=33 STYLE=\"width: 2.87in; height: 0.3in\"></P>");
		out.println("</TD>");
		out.println("</TR>");
		out.println("<TR VALIGN=TOP>");
		out.println("<TD WIDTH=35%>");
		out.println("<P><FONT FACE=\"Liberation Sans, sans-serif\">File System Path to the Region Data File:</FONT></P>");
		out.println("</TD>");
		out.println("<TD WIDTH=65%>");
		out.println("<P><INPUT TYPE=TEXT NAME=\"pathToRegionDataFile\" SIZE=51 STYLE=\"width: 4.3in; height: 0.3in\"></P>");
		out.println("</TD>");
		out.println("</TR>");
		out.println("<TR VALIGN=TOP>");
		out.println("<TD WIDTH=35%>");
		out.println("<P><FONT FACE=\"Liberation Sans, sans-serif\">Create Order Purge Timer?</FONT></P>");
		out.println("</TD>");
		out.println("<TD WIDTH=65%>");
		out.println("<P><INPUT TYPE=RADIO NAME=\"TimerOption\" VALUE=\"true\" STYLE=\"width: 0.53in; height: 0.19in\">Yes ");
		out.println("<INPUT TYPE=RADIO NAME=\"TimerOption\" VALUE=\"false\" CHECKED STYLE=\"width: 0.53in; height: 0.19in\">No </P>");
		out.println("</TD>");
		out.println("</TR>");
		out.println("</TABLE>");
		out.println("<P><INPUT TYPE=SUBMIT NAME=\"Submit\" VALUE=\"Submit\" STYLE=\"width: 1.31in; height: 0.31in\"><INPUT TYPE=RESET NAME=\"Reset\" VALUE=\"Reset\" STYLE=\"width: 1.31in; height: 0.31in\"></P>");
		out.println("</FORM>");
		out.println("</body></html>");
		
		out.close();
		
	}
	
	public void doPost(HttpServletRequest request
			, HttpServletResponse response) throws ServletException, IOException {
		
		// Get the parameters
		
		int batchSize         = new Integer(request.getParameter("batchSize"));
		int numberOfCustomers = new Integer(request.getParameter("numberOfCustomers"));
		int numberOfProducts  = new Integer(request.getParameter("numberOfProducts"));
		
		String pathToRegionFile = request.getParameter("pathToRegionDataFile");
		
		boolean createOrderPurgeTimer = new Boolean(request.getParameter("TimerOption"));
		
		long distributionCenterCreateTime = 0
			, customerCreateTime = 0
			, productCreateTime = 0;
		
		ServletOutputStream out = response.getOutputStream();
		
		MultipartResponse multipartResponse = new MultipartResponse(response);
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("</body></html>");
		
		multipartResponse.endResponse();
		
		//  Create the distribution centers
		
		long startTime = System.currentTimeMillis();
		
		try {
			distributionCenterManager.createDistributionCenters(pathToRegionFile, batchSize);
		} catch (CreateDataException e) {
			logger.log(Level.ERROR, e.getMessage());
			logger.log(Level.ERROR, e.getStackTrace().toString());
			response.setStatus(500);
			sendClientErrorMessage(out, multipartResponse, e.getMessage());
			throw new ServletException("Failed to create the distribution centers", e);
		}
		
		long endTime = System.currentTimeMillis();
		
		distributionCenterCreateTime = endTime - startTime;
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("<p>Successfully created the distribution centers, in " + distributionCenterCreateTime + "ms!</p>");
		out.println("</body></html>");
		
		multipartResponse.endResponse();
		
		//  Create customers
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("<p>Successfully created the distribution centers, in " + distributionCenterCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfCustomers + " customers.</p>");
		out.println("</body></html>");
		
		multipartResponse.endResponse();
		
		startTime = System.currentTimeMillis();
		
		try {
			customerManager.createCustomers(numberOfCustomers, batchSize);
		} catch(CreateDataException e) {
			logger.log(Level.ERROR, e.getMessage());
			logger.log(Level.ERROR, e.getStackTrace().toString());
			response.setStatus(500);
			sendClientErrorMessage(out, multipartResponse, e.getMessage());
			throw new ServletException("Failed to create the customers", e);
		}
		
		endTime = System.currentTimeMillis();
		
		customerCreateTime = endTime - startTime;
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("<p>Successfully created the distribution centers, in " + distributionCenterCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfCustomers + " customers.</p>");
		out.println("<p>Successfully created " + numberOfCustomers + " customers, in " + customerCreateTime + "ms!</p>");
		out.println("</body></html>");
		
		multipartResponse.endResponse();
		
		//  Create the products
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("<p>Successfully created the distribution centers, in " + distributionCenterCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfCustomers + " customers.</p>");
		out.println("<p>Successfully created " + numberOfCustomers + " customers, in " + customerCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfProducts + " products.</p>");
		out.println("</body></html>");
		
		multipartResponse.endResponse();
		
		startTime = System.currentTimeMillis();
		
		try {
			productManager.createProducts(numberOfProducts, batchSize);
		} catch(CreateDataException e) {
			logger.log(Level.ERROR, e.getMessage());
			logger.log(Level.ERROR, e.getStackTrace().toString());
			response.setStatus(500);
			sendClientErrorMessage(out, multipartResponse, e.getMessage());
			throw new ServletException("Failed to create the products", e);
		}
		
		endTime = System.currentTimeMillis();
		
		productCreateTime = endTime - startTime;
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("<p>Successfully created the distribution centers, in " + distributionCenterCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfCustomers + " customers.</p>");
		out.println("<p>Successfully created " + numberOfCustomers + " customers, in " + customerCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfProducts + " products.</p>");
		out.println("<p>Successfully created " + numberOfProducts + " products, in " + productCreateTime + "ms!</p>");
		out.println("</body></html>");
		
		multipartResponse.endResponse();
			
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("<p>Successfully created the distribution centers, in " + distributionCenterCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfCustomers + " customers.</p>");
		out.println("<p>Successfully created " + numberOfCustomers + " customers, in " + customerCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfProducts + " products.</p>");
		out.println("<p>Successfully created " + numberOfProducts + " products, in " + productCreateTime + "ms!</p>");
		
		if (createOrderPurgeTimer) {
			out.println("<p>Creating the Order Purge Timer.</p>");
		} else {
			out.println("<p>Deleting the Order Purge Timer, if it exists.</p>");
		}
		
		out.println("</body></html>");
		
		multipartResponse.endResponse();
		
		try {
			orderManager.createOrderPurgeTimer(createOrderPurgeTimer);
		} catch(CreateDataException e) {
			logger.log(Level.ERROR, e.getMessage());
			logger.log(Level.ERROR, e.getStackTrace().toString());
			response.setStatus(500);
			sendClientErrorMessage(out, multipartResponse, e.getMessage());
			throw new ServletException("Failed to create the order purge timer.", e);
		}
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println("Creating the distribution centers.</p>");
		out.println("<p>Successfully created the distribution centers, in " + distributionCenterCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfCustomers + " customers.</p>");
		out.println("<p>Successfully created " + numberOfCustomers + " customers, in " + customerCreateTime + "ms!</p>");
		out.println("<p>Creating " + numberOfProducts + " products.</p>");
		out.println("<p>Successfully created " + numberOfProducts + " products, in " + productCreateTime + "ms!</p>");
		
		if (createOrderPurgeTimer) {
			out.println("<p>Creating the Order Purge Timer.</p>");
			out.println("<p>Successfully created the Order Purge Timer!");
		} else {
			out.println("<p>Deleting the Order Purge Timer, if it exists.</p>");
			out.println("<p>Successfully deleted the Order Purge Timer!");
		}
		
		out.println("</body></html>");
		
		multipartResponse.endResponse();

		//  End everything
		
		multipartResponse.finish();
		
	}
	
	private void sendClientErrorMessage(ServletOutputStream out, MultipartResponse multipartResponse, String message) throws IOException {
		
		multipartResponse.startResponse("text/html");
		
		out.println("<html><head><title>");
		out.println("Creating Data!</title></head>");
		out.println("<body><p>");
		out.println(message);
		out.println("</p><p>Check the log file for more details.</p>");
		out.println("</body></html>");
		
		multipartResponse.endResponse();
		multipartResponse.finish();
		
		return;
		
	}

}
