package services.web;

import javax.ejb.EJB;
import javax.servlet.http.HttpServlet;

import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jboss.logmanager.Level;
import org.jboss.logmanager.Logger;

import services.ejb.OrderInquiry;
import services.ejb.OrderManager;
import services.entities.Order;
import services.entities.OrderLine;
import services.exceptions.OrderEntryProcessFailedException;

/**
 * Servlet Class
 *
 * This NewOrderServlet provides the method for processing an HTTP post
 * with an HTML form for a new order.  It processes the form, doing basic
 * validation that products and quantities are entered, and then calls
 * the processOrder method of the OrderEntryProcessCoordinator Stateless
 * Session Bean.
 * 
 */
public class NewOrderServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;
	private static Logger logger = Logger.getLogger(NewOrderServlet.class.toString());

	@EJB(lookup="java:app/OrderManagerEJB/OrderManagerBean!services.ejb.OrderManager")
	private OrderManager orderManager = null;

	public NewOrderServlet() {

		super();

	}

	public void init(ServletConfig config) throws ServletException {

		super.init(config);

	}

	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		response.setContentType("text/html");
		PrintWriter out = response.getWriter();

		HttpSession session = request.getSession();

		OrderInquiry orderInquiry = null;

		orderInquiry = (OrderInquiry) session.getAttribute("sfsbOrderInquiry");

		if (orderInquiry != null) {
			orderInquiry.reset();
		}

		try {

			long customerId        = Integer.parseInt(request.getParameter("customerid"));

			String[] products      = request.getParameterValues("productid");
			String[] quantities    = request.getParameterValues("quantity");

			int numberOfProducts   = 0;
			int numberOfQuantities = 0;

			for (int i = 0; i < products.length; i++) {
				if (products[i].equals("")) {
					numberOfProducts = i++;
					break;
				}
				if (i == (products.length - 1)) {
					numberOfProducts = products.length;
				}
			}

			for (int i = 0; i < quantities.length; i++) {
				if (quantities[i].equals("")) {
					numberOfQuantities = i++;
					break;
				}
				if (i == (quantities.length - 1)) {
					numberOfQuantities = quantities.length;
				}
			}

			if (numberOfProducts != numberOfQuantities) {
				out.println("<html><head><title>");
				out.println("Error filling in Order Form");
				out.println("</title></head>");
				out.println("<body>");
				out.println("<p>");
				out.println("Hey, fill out the form correctly!<br>");
				out.println("Number of products entered: ");
				out.println(Integer.toString(numberOfProducts));
				out.println("</br><br>Number of Quantities entered: ");
				out.println(Integer.toString(numberOfQuantities));
				out.println("</br>");
				out.println("</p>");
			} else {

				List<OrderLine> orderLines = populateLines(numberOfProducts, products, quantities);

				out.println("<html><head><title>");
				out.println("Created a New Order");
				out.println("</title></head>");
				out.println("<body>");

				out.println("<h1>");
				out.println("New Order");
				out.println("</h1>");

				Order newOrder = orderManager.createOrder(customerId
						, orderLines);

				out.println("<p>");
				out.print("This is your order id: ");
				out.print(newOrder.getOrderId());
				out.println("</p>");
				out.println("<p>");
				out.print("The total amount of your order is: $");
				out.print(newOrder.getTotalOrderAmount());
				out.println("</p>");
				out.println("<p>");
				StringBuffer link = new StringBuffer(80);
				link.append("<a href=/Order/OrderInquiry?customerId=");
				link.append(Long.toString(newOrder.getCustomerId()));
				link.append(">Take me to my order history.</a>");
				out.println(link.toString());
			}

		} catch (NumberFormatException e) {
			logger.log(Level.ERROR, "Error Processing the incoming CustomerId: " + request.getParameter("customerid"));
			logger.log(Level.ERROR, e.getStackTrace().toString());
			response.setStatus(400);
			throw new ServletException(e);
		} catch (OrderEntryProcessFailedException e) {
			logger.log(Level.ERROR, "Creating the order failed.  Transaction has been rolled back.");
			logger.log(Level.ERROR, e.getStackTrace().toString());
			response.setStatus(500);
			throw new ServletException(e);
		} finally {
			out.println("</body></html>");
			out.close();
		}

	}

	private List<OrderLine> populateLines(int numberOfLines
			, String[] products
			, String[] quantities) {

		List<OrderLine> orderLines = new ArrayList<OrderLine>();

		for (int i = 0; i < numberOfLines; i++) {
			if (products[i].compareTo("") != 0) {
				OrderLine orderLine = new OrderLine();
				orderLine.setProductId(Integer.parseInt(products[i]));
				if (quantities[i].compareTo("") != 0) {
					orderLine.setQuantity(Integer.parseInt(quantities[i]));
					orderLines.add(orderLine);
				}
			}
		}

		return orderLines;

	}

}
